require 'redis'
require 'json'

module Redisque
  extend self

  attr_accessor :halting

  @options = {:address => '127.0.0.1', :port => 6379, :password => '', :db => 0}

  # Connect to Redis DB
  # Example:
  #   Redisque.connect(:password => 'secret', :db => 1)
  # Arguments:
  #   options: (Hash)
  def connect(options)
    @options.merge! options    
    @redis = Redis.new(@options)
  end

  # Add task to queue 
  # Example:
  #   Redisque.enqueue('tasks', {:id => 1, :email => 'my@email.com'})
  # Arguments:
  #   queue: (String)
  #   data: (Hash)
  def enqueue(queue, data)
    redis.lpush(queue, JSON(data))
  end

  # Run along the queue and execute tasks
  def workoff(queue, pause = 5, &block)
    @backup_queue = '*' << queue
    crash_check(queue)
    loop do        
      process(queue, &block)
      break if @halting
      # anyway, we need to get some sleep
      sleep pause 
    end     
  end

  # Return Redis connection
  # Arguments:
  #   options: (Hash)
  def redis(options = @options)
    @redis || connect(options) 
  end

  # Process the queue
  # Arguments:
  #   queue: (String)
  def process(queue)
    # wait for new value in `queue` for `timeout` time 
    # pop it and copy to bak queue named `*{queue}`
    value = redis.rpoplpush(queue, @backup_queue)
    # if we do have some valid value then launch the worker
    if value
      yield(JSON.parse(value))
      # Remove task from backup queue
      redis.rpop @backup_queue
    end
  end

  # Check if backup queue has any task
  # Arguments:
  #   queue: (String)
  def crash_check(queue)    
    redis.lrange(@backup_queue,  0, -1).each { |unfinished| redis.lpush(queue, unfinished) }
    redis.del(@backup_queue)
  end

end
